@@ -18,7 +18,9 @@ import (
 )
 
 const (
-	branchName = "main"
+	branchName      = "main"
+	dotGit          = "dev/.git"
+	dotGitSubmodule = "dev/.git/modules/submodule"
 )
 
 func TestEnabledGitNotFound(t *testing.T) {
@@ -88,23 +90,23 @@ func TestEnabledInWorktree(t *testing.T) {
 			WorkingFolderContent:  TestRootPath + "dev/worktree.git\n",
 			ExpectedWorkingFolder: TestRootPath + "dev/.git/worktrees/folder_worktree",
 			ExpectedRealFolder:    TestRootPath + "dev/worktree",
-			ExpectedRootFolder:    TestRootPath + "dev/.git",
+			ExpectedRootFolder:    TestRootPath + dotGit,
 		},
 		{
 			Case:                  "submodule",
 			ExpectedEnabled:       true,
 			WorkingFolder:         "./.git/modules/submodule",
-			ExpectedWorkingFolder: TestRootPath + "dev/.git/modules/submodule",
-			ExpectedRealFolder:    TestRootPath + "dev/.git/modules/submodule",
-			ExpectedRootFolder:    TestRootPath + "dev/.git/modules/submodule",
+			ExpectedWorkingFolder: TestRootPath + dotGitSubmodule,
+			ExpectedRealFolder:    TestRootPath + dotGitSubmodule,
+			ExpectedRootFolder:    TestRootPath + dotGitSubmodule,
 		},
 		{
 			Case:                  "submodule with root working folder",
 			ExpectedEnabled:       true,
-			WorkingFolder:         TestRootPath + "repo/.git/modules/submodule",
-			ExpectedWorkingFolder: TestRootPath + "repo/.git/modules/submodule",
-			ExpectedRealFolder:    TestRootPath + "repo/.git/modules/submodule",
-			ExpectedRootFolder:    TestRootPath + "repo/.git/modules/submodule",
+			WorkingFolder:         TestRootPath + dotGitSubmodule,
+			ExpectedWorkingFolder: TestRootPath + dotGitSubmodule,
+			ExpectedRealFolder:    TestRootPath + dotGitSubmodule,
+			ExpectedRootFolder:    TestRootPath + dotGitSubmodule,
 		},
 		{
 			Case:                  "submodule with worktrees",
@@ -126,12 +128,12 @@ func TestEnabledInWorktree(t *testing.T) {
 		},
 	}
 	fileInfo := &platform.FileInfo{
-		Path:         TestRootPath + "dev/.git",
+		Path:         TestRootPath + dotGit,
 		ParentFolder: TestRootPath + "dev",
 	}
 	for _, tc := range cases {
 		env := new(mock.MockedEnvironment)
-		env.On("FileContent", TestRootPath+"dev/.git").Return(fmt.Sprintf("gitdir: %s", tc.WorkingFolder))
+		env.On("FileContent", TestRootPath+dotGit).Return(fmt.Sprintf("gitdir: %s", tc.WorkingFolder))
 		env.On("FileContent", filepath.Join(tc.WorkingFolder, tc.WorkingFolderAddon)).Return(tc.WorkingFolderContent)
 		env.On("HasFilesInDir", tc.WorkingFolder, tc.WorkingFolderAddon).Return(true)
 		env.On("HasFilesInDir", tc.WorkingFolder, "HEAD").Return(true)
@@ -163,7 +165,7 @@ func TestEnabledInBareRepo(t *testing.T) {
 	}{
 		{
 			Case:            "Bare repo on main",
-			IsBare:          "true",
+			IsBare:          trueStr,
 			HEAD:            "ref: refs/heads/main",
 			ExpectedEnabled: true,
 			ExpectedHEAD:    "main",
@@ -174,7 +176,7 @@ func TestEnabledInBareRepo(t *testing.T) {
 		},
 		{
 			Case:            "Bare repo on main remote enabled",
-			IsBare:          "true",
+			IsBare:          trueStr,
 			HEAD:            "ref: refs/heads/main",
 			ExpectedEnabled: true,
 			ExpectedHEAD:    "main",
@@ -436,6 +438,8 @@ func TestSetGitStatus(t *testing.T) {
 		ExpectedUpstreamGone bool
 		ExpectedAhead        int
 		ExpectedBehind       int
+		Rebase               bool
+		Merge                bool
 	}{
 		{
 			Case: "all different options on working and staging, no remote",
@@ -535,6 +539,40 @@ func TestSetGitStatus(t *testing.T) {
 			ExpectedRef:          "branch-is-gone",
 			ExpectedUpstreamGone: true,
 		},
+		{
+			Case: "rebase with 2 merge conflicts",
+			Output: `
+			# branch.oid 1234567891011121314
+			# branch.head rework-git-status
+			# branch.upstream origin/rework-git-status
+			# branch.ab +0 -0
+			1 AA N...
+			1 AA N...
+			`,
+			ExpectedUpstream: "origin/rework-git-status",
+			ExpectedHash:     "1234567",
+			ExpectedRef:      "rework-git-status",
+			Rebase:           true,
+			ExpectedStaging:  &GitStatus{ScmStatus: ScmStatus{Unmerged: 2}},
+		},
+		{
+			Case: "merge with 4 merge conflicts",
+			Output: `
+			# branch.oid 1234567891011121314
+			# branch.head rework-git-status
+			# branch.upstream origin/rework-git-status
+			# branch.ab +0 -0
+			1 AA N...
+			1 AA N...
+			1 AA N...
+			1 AA N...
+			`,
+			ExpectedUpstream: "origin/rework-git-status",
+			ExpectedHash:     "1234567",
+			ExpectedRef:      "rework-git-status",
+			Merge:            true,
+			ExpectedStaging:  &GitStatus{ScmStatus: ScmStatus{Unmerged: 4}},
+		},
 	}
 	for _, tc := range cases {
 		env := new(mock.MockedEnvironment)
@@ -554,6 +592,10 @@ func TestSetGitStatus(t *testing.T) {
 		if tc.ExpectedStaging == nil {
 			tc.ExpectedStaging = &GitStatus{}
 		}
+		g.Rebase = tc.Rebase
+		g.Merge = tc.Merge
+		tc.ExpectedStaging.Formats = map[string]string{}
+		tc.ExpectedWorking.Formats = map[string]string{}
 		g.setGitStatus()
 		assert.Equal(t, tc.ExpectedStaging, g.Staging, tc.Case)
 		assert.Equal(t, tc.ExpectedWorking, g.Working, tc.Case)
@@ -605,6 +647,7 @@ func TestGitCleanSSHURL(t *testing.T) {
 		{Case: "rsync no port, no user", Expected: "https://host.xz/path/to/repo", Upstream: "rsync://host.xz/path/to/repo.git/"},
 		{Case: "git no port, no user", Expected: "https://host.xz/path/to/repo", Upstream: "git://host.xz/path/to/repo.git"},
 		{Case: "gitea no port, no user", Expected: "https://src.example.com/user/repo", Upstream: "_gitea@src.example.com:user/repo.git"},
+		{Case: "git@ with user", Expected: "https://github.com/JanDeDobbeleer/oh-my-posh", Upstream: "git@github.com:JanDeDobbeleer/oh-my-posh"},
 		{Case: "unsupported", Upstream: "\\test\\repo.git"},
 	}
 	for _, tc := range cases {
@@ -628,6 +671,7 @@ func TestGitUpstream(t *testing.T) {
 		{Case: "Bitbucket", Expected: "BB", Upstream: "bitbucket.org/test"},
 		{Case: "Azure DevOps", Expected: "AD", Upstream: "dev.azure.com/test"},
 		{Case: "Azure DevOps Dos", Expected: "AD", Upstream: "test.visualstudio.com"},
+		{Case: "CodeCommit", Expected: "AC", Upstream: "codecommit::eu-west-1://test-repository"},
 		{Case: "Gitstash", Expected: "G", Upstream: "gitstash.com/test"},
 		{Case: "My custom server", Expected: "CU", Upstream: "mycustom.server/test"},
 	}
@@ -642,6 +686,7 @@ func TestGitUpstream(t *testing.T) {
 			GitlabIcon:      "GL",
 			BitbucketIcon:   "BB",
 			AzureDevOpsIcon: "AD",
+			CodeCommit:      "AC",
 			GitIcon:         "G",
 			UpstreamIcons: map[string]string{
 				"mycustom.server": "CU",
@@ -940,6 +985,7 @@ func TestGitCommit(t *testing.T) {
 			ce:jan@ohmyposh.dev
 			at:1673176335
 			su:docs(error): you can't use cross segment properties
+			ha:1234567891011121314
 			`,
 			Expected: &Commit{
 				Author: &User{
@@ -952,6 +998,7 @@ func TestGitCommit(t *testing.T) {
 				},
 				Subject:   "docs(error): you can't use cross segment properties",
 				Timestamp: time.Unix(1673176335, 0),
+				Sha:       "1234567891011121314",
 			},
 		},
 		{
@@ -995,7 +1042,7 @@ func TestGitCommit(t *testing.T) {
 
 	for _, tc := range cases {
 		env := new(mock.MockedEnvironment)
-		env.MockGitCommand("", tc.Output, "log", "-1", "--pretty=format:an:%an%nae:%ae%ncn:%cn%nce:%ce%nat:%at%nsu:%s")
+		env.MockGitCommand("", tc.Output, "log", "-1", "--pretty=format:an:%an%nae:%ae%ncn:%cn%nce:%ce%nat:%at%nsu:%s%nha:%H")
 		g := &Git{
 			scm: scm{
 				env:     env,
@@ -1006,3 +1053,106 @@ func TestGitCommit(t *testing.T) {
 		assert.Equal(t, tc.Expected, got, tc.Case)
 	}
 }
+
+func TestGitRemotes(t *testing.T) {
+	cases := []struct {
+		Case     string
+		Expected int
+		Config   string
+	}{
+		{
+			Case:     "Empty config file",
+			Expected: 0,
+		},
+		{
+			Case:     "Two remotes",
+			Expected: 2,
+			Config: `
+[remote "origin"]
+	url = git@github.com:JanDeDobbeleer/test.git
+	fetch = +refs/heads/*:refs/remotes/origin/*
+[remote "upstream"]
+	url = git@github.com:microsoft/test.git
+	fetch = +refs/heads/*:refs/remotes/upstream/*
+`,
+		},
+		{
+			Case:     "One remote",
+			Expected: 1,
+			Config: `
+[remote "origin"]
+	url = git@github.com:JanDeDobbeleer/test.git
+	fetch = +refs/heads/*:refs/remotes/origin/*
+`,
+		},
+		{
+			Case:     "Broken config",
+			Expected: 0,
+			Config:   "{{}}",
+		},
+	}
+
+	for _, tc := range cases {
+		env := new(mock.MockedEnvironment)
+		env.On("FileContent", "config").Return(tc.Config)
+
+		g := &Git{
+			scm: scm{
+				props:   properties.Map{},
+				realDir: "foo",
+				env:     env,
+			},
+		}
+
+		got := g.Remotes()
+		assert.Equal(t, tc.Expected, len(got), tc.Case)
+	}
+}
+
+func TestGitRepoName(t *testing.T) {
+	cases := []struct {
+		Case       string
+		Expected   string
+		WorkingDir string
+		RealDir    string
+		IsWorkTree bool
+	}{
+		{
+			Case:       "In worktree",
+			Expected:   "oh-my-posh",
+			IsWorkTree: true,
+			WorkingDir: "/Users/jan/Code/oh-my-posh/.git/worktrees/oh-my-posh2",
+		},
+		{
+			Case:       "Not in worktree",
+			Expected:   "oh-my-posh",
+			IsWorkTree: false,
+			RealDir:    "/Users/jan/Code/oh-my-posh",
+		},
+		{
+			Case:       "In worktree, unexpected dir",
+			Expected:   "",
+			IsWorkTree: true,
+			WorkingDir: "/Users/jan/Code/oh-my-posh2",
+		},
+	}
+
+	for _, tc := range cases {
+		env := new(mock.MockedEnvironment)
+		env.On("PathSeparator").Return("/")
+		env.On("GOOS").Return(platform.LINUX)
+
+		g := &Git{
+			scm: scm{
+				props:      properties.Map{},
+				env:        env,
+				realDir:    tc.RealDir,
+				workingDir: tc.WorkingDir,
+			},
+			IsWorkTree: tc.IsWorkTree,
+		}
+
+		got := g.repoName()
+		assert.Equal(t, tc.Expected, got, tc.Case)
+	}
+}
