@@ -8,7 +8,6 @@ import (
 	"github.com/jandedobbeleer/oh-my-posh/src/platform"
 	"github.com/jandedobbeleer/oh-my-posh/src/properties"
 
-	"github.com/shirou/gopsutil/v3/cpu"
 	"github.com/stretchr/testify/assert"
 )
 
@@ -75,12 +74,6 @@ func TestSysInfo(t *testing.T) {
 				},
 			},
 		},
-		{
-			Case:           "2 physical cpus",
-			ExpectedString: "1200 1200",
-			Template:       "{{range $cpu := .CPU}}{{round $cpu.Mhz 2 }} {{end}}",
-			SysInfo:        platform.SystemInfo{CPU: []cpu.InfoStat{{Mhz: 1200}, {Mhz: 1200}}},
-		},
 	}
 
 	for _, tc := range cases {
